﻿local addon, addonData = ...

local function GetDistance(x, y)
    local startX, startY = UnitPosition("player")
    local dX = startX - x
    local dY = startY - y
    return (dX * dX + dY * dY) ^ 0.5
end

local function GetClosestTreasure()
    local dist = 99999999
    local closest = nil
    for k,v in pairs(addonData.treasures) do
        local dist2 = GetDistance(v[3], v[4])
        if dist2 < dist and (v[7] == 0 or not IsQuestFlaggedCompleted(v[7])) then
            dist = dist2
            closest = v
        end
    end
    return closest, dist
end

if DBM then
    addonData.arrowAvailable = true
    if DBMArrow then
        local text = DBMArrow:CreateFontString(nil, "OVERLAY", "GameTooltipText")
        text:SetWidth(150)
        text:SetHeight(30)
        text:SetPoint("CENTER", DBMArrow, "CENTER", 0, -35)
        text:SetTextColor(1, 1, 1, 1)
        text:SetJustifyH("CENTER")
        DBMArrow.text = text
        addonData.arrowTextAvailable = true
    end
end

local function ShowArrow(x, y, text)
    if addonData.arrowAvailable then
        DBM.Arrow:ShowRunTo(x,y,nil,5)
    end
    if addonData.arrowTextAvailable then
        if not DBMArrow.text:IsShown() then
            DBMArrow.text:Show()
        end
        DBMArrow.text:SetText(text)
    end
end

local function HideArrow()
    if addonData.arrowAvailable then
        DBM.Arrow:Hide()
    end
    if addonData.arrowTextAvailable then
        DBMArrow.text:Hide()
    end
end

local interval = 1

local function onUpdate(self,elapsed)
    if not addonData.arrowEnabled then return end
    local _, _, _, mapid = UnitPosition("player")
    if not addonData.debug and mapid ~= 1116 then return end
    interval = interval + elapsed
    if interval >= 1 then
        interval = 0
        local closest, dist = GetClosestTreasure()
        if not closest then return end
        ShowArrow(closest[3],closest[4], format("%s\n%.02f yd", closest[5], dist))
    end
end

addonData.frame = CreateFrame("Frame")
addonData.frame:SetScript("OnUpdate", onUpdate)
addonData.enabled = true
addonData.arrowEnabled = true
addonData.debug = false

SLASH_DRTS1 = "/drts";

SlashCmdList["DRTS"] = function(msg, editBox)
    if msg:lower() == "on" then
        addonData.frame:SetScript("OnUpdate", onUpdate)
        addonData.enabled = true
        DEFAULT_CHAT_FRAME:AddMessage("DraenorTreasures enabled")
    elseif msg:lower() == "off" then
        addonData.frame:SetScript("OnUpdate", nil)
        addonData.enabled = false
        HideArrow()
        DEFAULT_CHAT_FRAME:AddMessage("DraenorTreasures disabled")
    elseif msg:lower() == "arrow" then
        if not addonData.arrowAvailable then
            DEFAULT_CHAT_FRAME:AddMessage("DraenorTreasures arrow not available. Please install DBM in order to use arrow.")
            return
        end
        addonData.arrowEnabled = not addonData.arrowEnabled
        DEFAULT_CHAT_FRAME:AddMessage("DraenorTreasures arrow "..(addonData.arrowEnabled and "enabled" or "disabled"))
        if not addonData.arrowEnabled then
            HideArrow()
        end
    elseif msg:lower() == "debug" then
        addonData.debug = not addonData.debug
        DEFAULT_CHAT_FRAME:AddMessage("DraenorTreasures debug "..(addonData.debug and "enabled" or "disabled"))
    else
        DEFAULT_CHAT_FRAME:AddMessage("Usage: /drts on|off")
        DEFAULT_CHAT_FRAME:AddMessage("Usage: /drts arrow (toggle)")
        DEFAULT_CHAT_FRAME:AddMessage("Usage: /drts debug (toggle)")
    end
end

local function WorldPosToScreenPos(x, y, locLeft, locRight, locTop, locBottom)
    local h, w = abs(locRight - locLeft), abs(locBottom - locTop);
    x = (locTop - x) / w;
    y = (locLeft - y) / h;
    return y, x;
end

local function TreasureMap_ResetPOI(button)
    button:SetWidth(32);
    button:SetHeight(32);
    button.Texture:SetWidth(16);
    button.Texture:SetHeight(16);
    button.Texture:SetPoint("CENTER", 0, 0);
    button.Texture:SetTexture("Interface\\Minimap\\POIIcons");

    button.specialPOIInfo = nil;
end

local debugMsgFormat = "Treasure %s (%u):\nzone %u\nxpos %f\nypos %f\ntracking quest %u (%s)\ntreasuremap quest %u (%s)\ndistance %.02f";

local function TreasureMapPOI_OnClick(self, button)
    if button == "RightButton" then
        WorldMapButton_OnClick(WorldMapButton, button);
        return;
    end
    if not addonData.debug then return end
    local t = addonData.treasures[self.id];
    local trquest, trstate = t[7], t[7] ~= 0 and (IsQuestFlaggedCompleted(t[7]) and "Completed" or "Not completed") or "No quest";
    local tmquest, tmstate = t[8], t[8] ~= 0 and (IsQuestFlaggedCompleted(t[8]) and "Completed" or "Not completed") or "No quest";
    print(format(debugMsgFormat, t[5], t[1], t[2], t[3], t[4], trquest, trstate, tmquest, tmstate, GetDistance(t[3], t[4])));
end

local function TreasureMap_CreatePOI(index)
    local button = CreateFrame("Button", "TreasureMapFramePOI"..index, WorldMapButton);
    button:RegisterForClicks("LeftButtonUp", "RightButtonUp");
    button:SetScript("OnEnter", WorldMapPOI_OnEnter);
    button:SetScript("OnLeave", WorldMapPOI_OnLeave);
    button:SetScript("OnClick", TreasureMapPOI_OnClick);

    button.Texture = button:CreateTexture(button:GetName().."Texture", "BACKGROUND");

    TreasureMap_ResetPOI(button);
    return button;
end

local function DrawTreasurePOI()
    local worldMapID = GetCurrentMapAreaID();
    local mapID, areaID, displayMapID, locLeft, locRight, locTop, locBottom, minLevel, maxLevel, flags = GetAreaMapInfo(worldMapID);

    for k,v in pairs(addonData.treasures) do
        local treasureMapPOI = _G["TreasureMapFramePOI"..k] or TreasureMap_CreatePOI(k);
        if v[2] ~= 0 and v[2] == areaID and (v[7] == 0 or not IsQuestFlaggedCompleted(v[7])) and (v[8] == 0 or not IsQuestFlaggedCompleted(v[8])) then
            local x, y = WorldPosToScreenPos(v[3], v[4], locLeft, locRight, locTop, locBottom);

            local name;
            if addonData.debug then
                name = format("%s (%u, %.02f, %.02f)", v[5], v[1], x * 100, y * 100);
            else
                name = format("%s (%.02f, %.02f)", v[5], x * 100, y * 100);
            end

            local description = v[6] ~= "" and v[6] or "Draenor Treasure!";

            x = x * WorldMapButton:GetWidth();
            y = -y * WorldMapButton:GetHeight();
            treasureMapPOI:SetPoint("CENTER", "WorldMapButton", "TOPLEFT", x, y);

            TreasureMap_ResetPOI(treasureMapPOI);

            local x1, x2, y1, y2 = GetPOITextureCoords(197);

            treasureMapPOI.Texture:SetTexCoord(x1, x2, y1, y2);

            treasureMapPOI.id = k;
            treasureMapPOI.name = name;
            treasureMapPOI.description = description;
            treasureMapPOI.mapLinkID = nil;

            treasureMapPOI:Show();
        else
            treasureMapPOI:Hide();
        end
    end
end

local function MyWorldMapFrame_OnUpdate(self, e)
    WorldMapFrame_OnUpdate(self, e);
    if addonData.enabled then
        DrawTreasurePOI();
    end
end

WorldMapFrame:SetScript("OnUpdate", MyWorldMapFrame_OnUpdate);
